#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# This file is part of SysPass Client
#
# Copyright (C) 2020  DigDeo SAS
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# python 3 headers, required if submitting to Ansible

import syspassclient
from colorama import init, Fore, Style

init(autoreset=True)
import warnings
from urllib3.exceptions import HTTPWarning, InsecureRequestWarning

warnings.simplefilter("ignore", HTTPWarning)
warnings.simplefilter("ignore", InsecureRequestWarning)
import requests

requests.urllib3.disable_warnings(category=InsecureRequestWarning)

from yaml import dump

try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper


import threading

lock = threading.Lock()


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            with lock:
                if cls not in cls._instances:
                    cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singleton):
    def __init__(self):
        syspassclient.Object.__init__(self)
        syspassclient.Utils.__init__(self)

        # Private property's
        self.__authToken = None
        self.__api_url = None
        self.__tokenPass = None
        self.__r_id = None

        # Initialize
        self.api_url = syspassclient.Config().api_url
        self.authToken = syspassclient.Config().authToken
        self.tokenPass = syspassclient.Config().tokenPass
        self.debug = syspassclient.Config().debug
        self.debug_level = syspassclient.Config().debug_level
        self.verbose = syspassclient.Config().verbose
        self.verbose_level = syspassclient.Config().verbose_level

        self.r_id = 1

    @property
    def api_url(self):
        """
        The url of the SysPass API

        :return: the url of the SysPass API or None if not set
        :rtype: str or None
        """
        return self.__api_url

    @api_url.setter
    def api_url(self, api_url=None):
        """
        Set ``api_url`` property

        :param api_url: A SysPass API Key
        :type api_url: str or None
        :raise TypeError: if ``api_url`` is not a str of None
        """

        if api_url is None:
            if self.__api_url is not None:
                self.__api_url = None
            return

        self.is_str_or_raise(api_url)
        self.is_url_or_raise(api_url)

        if self.__api_url != api_url:
            self.__api_url = api_url

    @property
    def authToken(self):
        """
        Whether the application will paint directly on the widget.

        :return: True or False
        :rtype: bool
        """
        return self.__authToken

    @authToken.setter
    def authToken(self, authToken=None):
        """
        Set ``api_key`` property

        :param authToken: A SysPass API Key
        :type authToken: str or None
        :raise TypeError: if ``api_key`` is not a str of None
        """
        if authToken is None or authToken == "":
            if self.__authToken != authToken:
                self.__authToken = None
            return

        self.is_str_or_raise(authToken)
        self.is_ascii_or_raise(authToken)

        if self.__authToken != authToken:
            self.__authToken = authToken

    @property
    def tokenPass(self):
        """
        Whether the application will paint directly on the widget.

        :return: True or False
        :rtype: bool
        """
        return self.__tokenPass

    @tokenPass.setter
    def tokenPass(self, tokenPass=None):
        """
        Set ``api_acc_tokpwd`` property

        :param tokenPass: A SysPass API Key
        :type tokenPass: str or None
        :raise TypeError: if ``api_acc_tokpwd`` is not a str of None
        """
        if tokenPass is None or tokenPass == "":
            if self.__tokenPass != tokenPass:
                self.__tokenPass = None
            return

        if tokenPass is not None:
            self.is_str_or_raise(tokenPass)
            self.is_ascii_or_raise(tokenPass)

        if self.__tokenPass != tokenPass:
            self.__tokenPass = tokenPass

    def make_post_request(self, data=None, verify=False):
        """
        make the request by 'request' function.

        :param data: the data it will be convert on json
        :type data: dict
        :param verify: ssl certificate verification
        :type verify: bool
        :return: the request.post return
        :rtype: dict
        """
        # if self.debug and self.debug_level > 0:
        #     class_name = "{0} ".format(self.__class__.__name__.upper())
        #     action_name = "| {1} | {0} ".format("request >".lower(), data['method'])
        #
        #     sys.stdout.write("\u001b[33m")
        #     sys.stdout.write("{0}".format(class_name))
        #     sys.stdout.write("\u001b[0m")
        #     sys.stdout.write("{0}".format(action_name))
        #     sys.stdout.write("\n")
        #     sys.stdout.write("{0}".format(dump(data)))
        #     sys.stdout.write("\u001b[0m")
        #     sys.stdout.write("\n")
        #
        #     sys.stdout.flush()
        if self.verbose:
            print(Fore.YELLOW + Style.BRIGHT + "{0}: ".format(self.__class__.__name__.upper()), end='')
            print(Fore.WHITE + Style.BRIGHT + "{0} ".format(data['method']))
            for line in dump(data).split("\n"):
                if line is not None and line != '':
                    print(Fore.GREEN + Style.BRIGHT + "> " + Fore.RESET + Style.RESET_ALL + str(line))
            print(Fore.RESET + Style.RESET_ALL, end='')

        request = requests.post(url=self.api_url, json=data, verify=verify)
        # print(request.text)
        from json.decoder import JSONDecodeError
        try:
            return request.json()
        except JSONDecodeError:
            return request.text

        # return requests.post(url=self.api_url, json=data, verify=verify).json()

    def increase_request_id(self, increment=1):
        """
        In case of multi request, request ID must be increase for permit to the API work with.

        :param increment: a increment value
        :type increment: int
        :raise TypeError: if ``increment`` is not a int
        :raise ValueError: if ``increment`` is not a positive value
        """
        self.is_int_or_raise(increment)

        if increment <= 0:
            raise ValueError("'increment' must be a positive value")

        self.r_id += increment

    def generate_json(self, **parameters):

        if "method" not in parameters:
            raise (KeyError('"parameters" must have a key name "method"'))
        method = parameters["method"]
        del parameters["method"]

        # workaround about 'pass' python world reserved
        if "password" in parameters:
            password = parameters["password"]
            del parameters["password"]
            parameters["pass"] = password

        # workaround for 'global' world reserved
        if "Global" in parameters:
            Global = parameters["Global"]
            del parameters["Global"]
            parameters["global"] = Global

        # workaround for 'id' world reserved
        if "ugid" in parameters:
            ugid = parameters["ugid"]
            del parameters["ugid"]
            parameters["id"] = ugid
        if "cid" in parameters:
            cid = parameters["cid"]
            del parameters["cid"]
            parameters["id"] = cid
        if "tagid" in parameters:
            tagid = parameters["tagid"]
            del parameters["tagid"]
            parameters["id"] = tagid
        if "account_id" in parameters:
            account_id = parameters["account_id"]
            del parameters["account_id"]
            parameters["id"] = account_id

        for key, value in dict(parameters).items():
            if value is None:
                del parameters[key]
            if value is []:
                del parameters[key]

        data = {"jsonrpc": "2.0", "method": method, "params": parameters, "id": self.r_id}
        return data

    def print_returned_value(self, req):
        if self.verbose:
            if self.verbose_level > 1:
                print(Fore.YELLOW + Style.BRIGHT + "{0}: ".format(self.__class__.__name__.upper()), end='')
                print(Fore.WHITE + Style.BRIGHT + "Returned Value")

            if 'error' in req:
                for line in dump(req, allow_unicode=True).split("\n"):
                    if line is not None and line != '':
                        print(Fore.CYAN + Style.BRIGHT + "< " + Fore.RED + Style.BRIGHT + str(line))
            else:
                for line in dump(req, allow_unicode=True).split("\n"):
                    if line is not None and line != '':
                        print(Fore.CYAN + Style.BRIGHT + "< " + Fore.RESET + Style.RESET_ALL + str(line))
            print(Fore.RESET + Style.RESET_ALL, end='')

    def account_search(
            self,
            method="account/search",
            authToken=None,
            text=None,
            count=None,
            categoryId=None,
            clientId=None,
            tagsId=None,
            op=None,
            matchall=None
    ):
        """
        Search for accounts:

        method: account/search

        Parameter 	Type 	Required 	Description
        authToken 	string 	yes 	 User’s API token
        text 	    string 	no 	     Text to search for
        count 	    int 	no 	     Number of results to display
        categoryId 	int 	no 	     Category’s Id for filtering
        clientId 	int 	no 	     Client’s Id for filtering
        tagsId 	    array 	no 	     Tags’ Id for filtering
        op 	        string 	no 	     Operator used for filtering. It can be either ‘or’ or ‘and’

        parameter ``match_all`` is not on the original API

        :param method: the method name
        :type method: str
        :param authToken: User’s API token
        :type authToken: str
        :param text: Text to search for
        :type text: str or None
        :param count: Number of results to display
        :type count: int or None
        :param categoryId: Category’s Id for filtering
        :type categoryId: int or None
        :param clientId: Client’s Id for filtering
        :type clientId: int or None
        :param tagsId: Tags’ Id for filtering
        :type tagsId: array or None
        :param op: Operator used for filtering. It can be either ‘or’ or ‘and’
        :type: op: str or None
        :param matchall:
        :type matchall:
        :return: Return the searching result
        :rtype: str in case of success or None if nothing found
        """
        if authToken is None:
            authToken = self.authToken

        # Exit as soon of possible
        self.look_for_args(
            method=method,
            authToken=authToken,
            text=text,
            count=count,
            categoryId=categoryId,
            clientId=clientId,
            tagsId=tagsId,
            op=op,
        )

        data = self.generate_json(
            method=method,
            authToken=authToken,
            text=text,
            count=count,
            categoryId=categoryId,
            clientId=clientId,
            tagsId=tagsId,
            op=op,
        )

        self.increase_request_id()

        req = self.make_post_request(data)

        self.print_returned_value(req)

        # look_for_error(data, req)

        if 'result' in req and 'count' in req['result'] and req['result']['count'] > 0:
            if matchall is None:
                for res in req['result']['result']:
                    if res['name'] == text:
                        return req['result']['result'][0]['id']
            else:
                return req['result']['result']
        # elif 'error' in req:
        #     raise AnsibleError('AccountSearch Error : %s' % req)
        else:
            return None

    def account_delete(self, method="account/delete", authToken=None, account_id=None):
        """
        Delete an account

        method: account/delete

        Parameter 	Type 	Required 	Description
        authToken 	string 	yes 	    User’s API token
        id 	        int 	yes 	    Account’s Id

        :param method: must be 'account/delete'
        :type method: str
        :param authToken: User’s API token
        :type authToken: str
        :param account_id: Account’s Id
        :type account_id: int
        """
        if authToken is None:
            authToken = self.authToken

        # Exit as soon of possible
        self.look_for_args(method=method, authToken=authToken, account_id=account_id)

        data = self.generate_json(method=method, authToken=authToken, account_id=account_id)

        self.increase_request_id()

        req = self.make_post_request(data)

        self.print_returned_value(req)

        self.look_for_error(data, req)

        if req["result"]["resultCode"] == 0:
            return req["result"]
        # else:
        #     raise AnsibleError("AccountDelete Error : %s" % req)

    def account_view(self, method="account/view", authToken=None, tokenPass=None, account_id=None):
        """
        Search for accounts:

        method: account/search

        Parameter 	Type 	Required 	Description
        authToken 	string 	yes 	    User’s API token
        tokenPass 	string 	yes 	    API token’s pass
        id 	        int 	yes 	    Account’s Id

        :param method: the method name
        :type method: str
        :param authToken: User’s API token
        :type authToken: str
        :param tokenPass: API token’s pass, is used to decrypt encrypted data.
        :type tokenPass: str
        :param account_id:  Account’s Id
        :type account_id: int
        """
        if authToken is None:
            authToken = self.authToken

        if tokenPass is None:
            tokenPass = self.tokenPass

        self.look_for_args(method=method, authToken=authToken, tokenPass=tokenPass, account_id=account_id)

        data = self.generate_json(method=method, authToken=authToken, tokenPass=tokenPass, account_id=account_id)

        self.increase_request_id()

        req = self.make_post_request(data)

        self.print_returned_value(req)

        self.look_for_error(data, req)

        if 'result' in req and 'result' in req['result'] and len(req['result']['result']) > 0:
            return req["result"]["result"]
        else:
            return None

    def account_viewpass(self, method="account/viewPass", authToken=None, tokenPass=None, account_id=None,
                         details=None):
        """
        Get account’s password

        Parameter 	Type 	Required 	Description
        authtoken 	string 	yes 	    User’s API token
        tokenpass 	string 	yes 	    API token’s pass
        account_id 	int 	yes 	    Account’s Id
        details 	int 	no 	        Whether to return account’s details within response

        method: account/viewPass

        :param method: the method name
        :type method: str
        :param authToken: User’s API token
        :type authToken: str
        :param tokenPass: API token’s pass, is used to decrypt encrypted data.
        :type tokenPass: str
        :param account_id: Account’s Id
        :type account_id: int
        :param details: Whether to return account’s details within response
        :type details: int or None
        """
        if authToken is None:
            authToken = self.authToken

        if tokenPass is None:
            tokenPass = self.tokenPass

        self.look_for_args(
            method=method,
            authToken=authToken,
            tokenPass=tokenPass,
            account_id=account_id,
            details=details
        )

        data = self.generate_json(
            method=method,
            authToken=authToken,
            tokenPass=tokenPass,
            account_id=account_id,
            details=details
        )

        self.increase_request_id()

        req = self.make_post_request(data)

        self.print_returned_value(req)

        self.look_for_error(data, req)

        # We got a response
        if 'result' in req and 'count' in req['result'] and req['result']['count'] > 0:
            return req['result']['result']['password']
        # else:
        #     raise AnsibleError('AccountViewpass Error : %s' % req)

    def account_create(
            self,
            method="account/create",
            authToken=None,
            tokenPass=None,
            name=None,
            categoryId=None,
            clientId=None,
            password=None,
            tagsId=None,
            userGroupId=None,
            parentId=None,
            login=None,
            url=None,
            notes=None,
            private=None,
            privateGroup=None,
            expireDate=None,
    ):
        """
        Create account

        account/create

        Parameter 	    Type 	Required 	Description
        authToken 	    string 	yes 	    User’s API token
        tokenPass 	    string 	yes 	    API token’s pass
        name 	        string 	yes 	    Account’s name
        categoryId 	    int 	yes 	    Account’s category Id
        clientId 	    int 	yes 	    Account’s client Id
        pass 	        string 	yes 	    Account’s password
        tagsId 	        array 	no 	        Account’s tags Id
        userGroupId     int 	no 	        Account’s user group Id
        parentId 	    int 	no 	        Account’s parent Id
        login 	        string 	no 	        Account’s login
        url 	        string 	no 	        Account’s access URL or IP
        notes 	        string 	no 	        Account’s notes
        private 	    int 	no 	        Set account as private. It can be either 0 or 1
        privateGroup 	int 	no 	        Set account as private for group. It can be either 0 or 1
        expireDate 	    int 	no 	        Expire date in UNIX timestamp format

        Note: pass have been change by password, due that a python reserved world. That is the json request creator it
        change password for pass just at end of the process.
        
        :param method: must be account/create.
        :type method: str
        :param authToken: User’s API token
        :type authToken: str
        :param tokenPass: API token’s pass
        :type tokenPass: str
        :param name: Account’s name
        :type name: str
        :param categoryId: Account’s category Id
        :type categoryId: int
        :param clientId: Account’s client Id
        :type clientId: int
        :param password: Account’s password
        :type password: str
        :param tagsId: Account’s tags Id
        :type tagsId: list
        :param userGroupId: Account’s user group Id
        :type userGroupId: int
        :param parentId: Account’s parent Id
        :type parentId: int or None
        :param login: Account’s login
        :type login: str
        :param url: Account’s access URL or IP
        :type url: str
        :param notes: Account’s notes
        :type notes: str
        :param private: Set account as private. It can be either 0 or 1
        :type private: int
        :param privateGroup: Set account as private for group. It can be either 0 or 1
        :type privateGroup: int
        :param expireDate: Expire date in UNIX timestamp format
        :type expireDate: int
        """
        if authToken is None:
            authToken = self.authToken
        if authToken is None:
            raise TypeError("'authToken' must be set")

        if tokenPass is None:
            tokenPass = self.tokenPass

        if tokenPass is None:
            raise TypeError("'tokenPass' must be set")
        if expireDate == "":
            expireDate = None

        self.look_for_args(
            method=method,
            authToken=authToken,
            tokenPass=tokenPass,
            name=name,
            categoryId=categoryId,
            clientId=clientId,
            password=password,
            tagsId=tagsId,
            userGroupId=userGroupId,
            parentId=parentId,
            login=login,
            url=url,
            notes=notes,
            private=private,
            privateGroup=privateGroup,
            expireDate=expireDate,
        )

        data = self.generate_json(
            method=method,
            authToken=authToken,
            tokenPass=tokenPass,
            name=name,
            categoryId=categoryId,
            clientId=clientId,
            password=password,
            tagsId=tagsId,
            userGroupId=userGroupId,
            parentId=parentId,
            login=login,
            url=url,
            notes=notes,
            private=private,
            privateGroup=privateGroup,
            expireDate=expireDate,
        )
        self.increase_request_id()

        req = self.make_post_request(data)

        self.print_returned_value(req)

        self.look_for_error(data, req)

        if req["result"]["itemId"] > 0:
            return req["result"]["itemId"]
        else:
            return None

    def category_search(self, method="category/search", authToken=None, text=None, count=None):
        """
        Searches syspass category.
        text is the keyword.
        count is the number of results.
        """
        if authToken is None:
            authToken = self.authToken

        self.look_for_args(method=method, authToken=authToken, text=text, count=count)

        data = self.generate_json(method=method, authToken=authToken, text=text, count=count)

        self.increase_request_id()

        req = self.make_post_request(data)

        self.print_returned_value(req)

        # look_for_error(data, req)

        # We got a response

        if 'result' in req and 'count' in req['result'] and req['result']['count'] > 0:
            for res in req['result']['result']:
                if res['name'] == text:
                    return req['result']['result'][0]['id']
        # elif 'error' in req:
        #     raise AnsibleError('CategorySearch Error : %s' % (req['error']))
        else:
            return None

    def category_create(self, method="category/create", authToken=None, name=None, description=None):
        """
        Create category

        Parameter 	Type 	Required 	Description
        method      string  yes         category/create
        authToken 	string 	yes 	    User’s API token
        name 	    string 	yes 	    Category’s name
        description string 	no 	        Category’s description

        :param method: must be category/create
        :type method: str
        :param authToken: User’s API token
        :type authToken: str
        :param name: Category’s name
        :type name: str
        :param description: Category’s description
        :type description: str
        """
        if authToken is None:
            authToken = self.authToken

        self.look_for_args(method=method, authToken=authToken, name=name, description=description)

        data = self.generate_json(method=method, authToken=authToken, name=name, description=description)

        self.increase_request_id()

        req = self.make_post_request(data)

        self.print_returned_value(req)

        # look_for_error(data, req)
        if 'result' in req and 'itemId' in req['result'] and req['result']['itemId'] > 0:
            if req["result"]['itemId'] is None:
                return self.category_search(text=name)
            else:
                return req["result"]['itemId']
        else:
            return self.category_search(text=name)

    def category_delete(self, method="category/delete", authToken=None, cid=None):
        """
        category/delete

        Delete category

        Parameter 	Type 	Required 	Description
        authToken 	string 	yes 	    User’s API token
        cid 	    int 	yes 	    Category’s Id
        """
        if authToken is None:
            authToken = self.authToken

        self.look_for_args(method=method, authToken=authToken, cid=cid)

        data = self.generate_json(method=method, authToken=authToken, cid=cid)

        self.increase_request_id()

        req = self.make_post_request(data)

        self.print_returned_value(req)

        self.look_for_error(data, req)

        # print(req)
        if req["result"]["resultCode"] == 0:
            return req["result"]
        # else:
        #     raise AnsibleError("CategoryDelete Error : %s" % req)

    def client_search(self, method="client/search", authToken=None, text=None, count=None):
        """
        client/search

        Search for clients
        Parameter 	Type 	Required 	Description
        authToken 	string 	yes 	    User’s API token
        text 	    string 	no 	        Text to search for
        count 	    int 	no 	        Number of results to display
        """
        if authToken is None:
            authToken = self.authToken

        self.look_for_args(method=method, authToken=authToken, text=text, count=count)

        data = self.generate_json(method=method, authToken=authToken, text=text, count=count)

        self.increase_request_id()

        req = self.make_post_request(data)

        self.print_returned_value(req)

        # look_for_error(data, req)

        # We got a response
        if 'result' in req and 'count' in req['result'] and req['result']['count'] > 0:
            for res in req['result']['result']:
                if res['name'].upper() == text.upper():
                    return req['result']['result'][0]['id']
        # elif 'error' in req:
        #     raise AnsibleError('ClientSearch Error : %s' % (req['error']))
        else:
            return None

    def client_create(self, method="client/create", authToken=None, name=None, description=None, Global=False):
        """
        Creates a syspass client.

        :param method: must be 'client/create'
        :type method: str
        :param authToken: User’s API token
        :type authToken: str
        :param name: Client’s name
        :type name: str
        :param description: Client’s description
        :type description: str
        :param Global: Set client as global. It can be either 0 or 1
        :type Global: int
        """
        if authToken is None:
            authToken = self.authToken

        if Global is not None:
            Global = int(bool(Global))

        self.look_for_args(method=method, authToken=authToken, name=name, description=description, Global=Global)

        data = self.generate_json(method=method, authToken=authToken, name=name, description=description, Global=Global)

        self.increase_request_id()

        req = self.make_post_request(data)

        self.print_returned_value(req)

        # look_for_error(data, req)

        if 'result' in req and 'itemId' in req['result'] and req['result']['itemId'] > 0:
            return req['result']['itemId']

        return self.client_search(text=name)

    def client_delete(self, method='client/delete', authToken=None, cid=None):
        """
        client/delete

        Delete client

        Parameter 	Type 	Required 	Description
        authToken 	string 	yes 	    User’s API token
        cid 	        int 	yes 	    Client’s Id

        :param method: must be 'client/create'
        :type method: str
        :param authToken: User’s API token
        :type: authToken: str
        :param cid: Client’s Id
        :type cid: int
        """
        if authToken is None:
            authToken = self.authToken

        self.look_for_args(method=method, authToken=authToken, cid=cid)

        data = self.generate_json(method=method, authToken=authToken, cid=cid)

        self.increase_request_id()

        req = self.make_post_request(data)

        self.print_returned_value(req)

        self.look_for_error(data, req)

        if 'result' in req and 'resultCode' in req['result'] and req['result']['resultCode'] == 0:
            return req['result']['resultCode']
        else:
            return None

    def tag_search(self, method='tag/search', authToken=None, text=None):
        """
        tag/search

        Search for tags

        Parameter 	Type 	Required 	Description
        authToken 	string 	yes 	    User’s API token
        name 	    string 	yes 	    Tag’s name

        :param method: must be "tag/search"
        :type method: str
        :param authToken: User’s API token
        :type authToken: str
        :param text: Tag’s name
        :type text: str
        """
        if authToken is None:
            authToken = self.authToken

        self.look_for_args(method=method, authToken=authToken, text=text)

        data = self.generate_json(method=method, authToken=authToken, text=text)

        self.increase_request_id()

        req = self.make_post_request(data)

        self.print_returned_value(req)

        # look_for_error(data, req)

        if 'result' in req and 'result' in req['result'] and len(req['result']['result']) > 0:
            for reponse in req["result"]["result"]:
                if 'name' in reponse and reponse['name'] == text:
                    return reponse['id']

        return None

    def tag_view(self, method="tag/view", authToken=None, tokenPass=None, tagid=None):
        """
        Tags

        tag/view

        Search for tags

        Parameter 	Type 	Required 	Description
        authToken 	string 	yes 	    User’s API token
        tokenPass 	string 	yes      	API token’s pass
        tagid 	    int 	yes 	    Number of results to display

        :param method: must be "tag/search"
        :type method: str
        :param authToken: User’s API token
        :type authToken: str
        :param tokenPass: Text to search for
        :type tokenPass: str
        :param tagid: Tag’s Id
        :type tagid: int
        """
        if authToken is None:
            authToken = self.authToken

        if tokenPass is None:
            tokenPass = self.tokenPass

        self.look_for_args(method=method, authToken=authToken, tokenPass=tokenPass, tagid=tagid)

        data = self.generate_json(method=method, authToken=authToken, tokenPass=tokenPass, tagid=tagid)

        self.increase_request_id()

        req = self.make_post_request(data)

        self.print_returned_value(req)

        self.look_for_error(data, req)

        if 'result' in req and 'result' in req['result'] and len(req['result']['result']) > 0:
            return req["result"]["result"]
        else:
            return None

    def tag_create(self, method='tag/create', authToken=None, name=None):
        """
        tag/create

        Create tag

        Parameter 	Type 	Required 	Description
        authToken 	string 	yes 	    User’s API token
        name 	    string 	yes 	    Tag’s name

        :param method: must be "tag/search"
        :type method: str
        :param authToken: User’s API token
        :type authToken: str
        :param name: Tag’s name
        :type name: str
        """
        if authToken is None:
            authToken = self.authToken

        self.look_for_args(method=method, authToken=authToken, name=name)

        data = self.generate_json(method=method, authToken=authToken, name=name)

        self.increase_request_id()

        req = self.make_post_request(data)

        self.print_returned_value(req)

        # look_for_error(data, req)

        if 'result' in req and 'itemId' in req['result'] and req['result']['itemId'] > 0:
            return req['result']['itemId']

        return self.tag_search(text=name)

    def tag_delete(self, method='tag/delete', authToken=None, tagid=None):
        """
        tag/delete

        Delete tag

        Parameter 	Type 	Required 	Description
        authToken 	string 	yes 	    User’s API token
        id 	        int 	yes 	    Tag’s Id

        :param method: must be 'tag/delete'
        :type method: str
        :param authToken: User’s API token
        :type authToken: str
        :param tagid: Tag’s Id
        :type tagid: int
        """
        if authToken is None:
            authToken = self.authToken

        self.look_for_args(method=method, authToken=authToken, tagid=tagid)

        data = self.generate_json(method=method, authToken=authToken, tagid=tagid)

        self.increase_request_id()

        req = self.make_post_request(data)

        self.print_returned_value(req)

        self.look_for_error(data, req)

        if 'result' in req and 'resultCode' in req['result']:
            return req['result']['resultCode']
        else:
            return None
        # if req['result']['resultCode'] == 0:
        #     return req['result']
        # else:
        #     raise AnsibleError('TagDelete Error : %s' % (req))

    def user_group_create(self, method="userGroup/create", authToken=None, name=None, description=None, usersId=None):
        """
        userGroup/create

        Create user group

        Parameter 	    Type 	Required 	Description
        authToken 	    string 	yes 	    User’s API token
        name 	        string 	yes 	    User group’s name
        description 	string 	no 	        User group’s description
        usersId 	    array 	no 	        User group’s users Id

        :param method: must be 'userGroup/create'
        :type method: str
        :param authToken: User’s API token
        :type authToken: str
        :param name: User group’s name
        :type name: str
        :param description: User group’s description
        :type description: str
        :param usersId: User group’s users Id
        :type usersId: list
        """
        if authToken is None:
            authToken = self.authToken

        self.look_for_args(method=method, authToken=authToken, name=name, description=description, usersId=usersId)
        data = self.generate_json(
            method=method, authToken=authToken, name=name, description=description, usersId=usersId
        )
        self.increase_request_id()
        req = self.make_post_request(data)
        self.print_returned_value(req)

        # look_for_error(data, req)
        if 'result' in req and 'itemId' in req['result'] and req['result']['itemId'] > 0:
            return req["result"]["itemId"]

        return self.user_group_search(text=name)

    def user_group_search(self, method="userGroup/search", authToken=None, text=None, count=None):
        """
        Search for user groups

        userGroup/search

        Parameter 	Type 	Required 	Description
        authToken 	string 	yes 	    User’s API token
        text    	string 	no 	        Text to search for
        count 	    int 	no 	        Number of results to display
        """

        if authToken is None:
            authToken = self.authToken

        self.look_for_args(method=method, authToken=authToken, text=text, count=count)
        data = self.generate_json(method=method, authToken=authToken, text=text, count=count)
        self.increase_request_id()
        req = self.make_post_request(data)
        self.print_returned_value(req)

        if 'result' in req and 'count' in req['result'] and req['result']['count'] > 0:
            for res in req['result']['result']:
                if res['name'].upper() == text.upper():
                    return req['result']['result'][0]['id']

        else:
            self.look_for_error(data, req)
            return None

    def user_group_delete(self,
                          method='userGroup/delete', authToken=None, ugid=None):
        """
        userGroup/delete

        Delete user group

        Parameter 	Type 	Required 	Description
        authToken 	string 	yes 	    User’s API token
        ugid 	    int 	yes 	    User group’s Id

        :param method: must be 'userGroup/delete'
        :type method: str
        :param authToken: User’s API token
        :type authToken: str
        :param ugid: User group’s Id
        :type ugid: int
        """
        if authToken is None:
            authToken = self.authToken

        self.look_for_args(method=method, authToken=authToken, ugid=ugid)
        data = self.generate_json(method=method, authToken=authToken, ugid=ugid)
        self.increase_request_id()
        req = self.make_post_request(data)
        self.print_returned_value(req)

        if 'result' in req and 'resultCode' in req['result'] and req['result']['resultCode'] == 0:
            return req['result']
        else:
            return None
            # look_for_error(data, req)

#
#     def Backup(self):
#         """
#         https://github.com/nuxsmin/sysPass/issues/1004#issuecomment-411487284
#         """
#         data = {"jsonrpc": "2.0",
#                 "method": "backup",
#                 "params": {
#                     "authToken": self.api_key,
#                 },
#                 "id": self.r_id
#                 }
#
#         self.r_id += 1
#         req = requests.post(self.api_url, json=data, verify=False)
#         if 'result' in req.json():
#             return req.json()['result']
#         else:
#             raise AnsibleError('Backup Error : %s' % (req.json()))
#
#



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# This file is part of SysPass Client
#
# Copyright (C) 2020  DigDeo SAS
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

import random
import string
import six
import syspassclient


class Utils(syspassclient.CheckType):
    def __init__(self):
        syspassclient.CheckType.__init__(self)

    def look_for_error(self, data, req):
        """
        Raise when a key name 'error' is found.

        The function try to make tests for know if the error message is come back with the a keys': \
        'message', 'code', 'data'

        :param data: a JSon text as originally send to SysPass
        :type: dict
        :param req: a JSon text as return by SysPass
        :type req: dict
        """
        self.is_dict_or_raise(data)
        self.is_dict_or_raise(req)
        if "error" in req:
            error_message = ""
            error_message += "method: "
            error_message += str(data["method"])
            if "message" in req["error"]:
                if req["error"]["message"] == "Internal error":
                    raise ValueError('syspass client have return a Internal error, all operation is stopped')
                # error_message += ", message: "
                # error_message += str(req["error"]["message"])
            # if "code" in req["error"]:
            #     error_message += ", code: "
            #     error_message += str(req["error"]["code"])
            # if "data" in req["error"]:
            #     error_message += ", data: "
            #     error_message += str(req["error"]["data"])
            # print(error_message)
            # raise AnsibleError("SysPass Error -> %s" % error_message)

    def look_for_args(self, **parameters):
        """
        Check Args for a method, it function try to Raise

        Test if the method exit on the API version

        #. Test if it have wrong parameters

        #. Test if every require parameters have been set

        #. Test the type of each parameter

        :param parameters: dict of parameters key=value
        :type parameters: keys
        """
        self.look_for_valid_method(parameters=parameters)
        self.look_for_parameters_injection(parameters=parameters)
        self.look_for_required_params(parameters=parameters)
        self.look_for_parameters_type(parameters=parameters)

    def look_for_parameters_type(self, **parameters):
        if "parameters" in parameters:
            parameters = parameters["parameters"]

        if "method" not in parameters:
            raise (KeyError('"parameters" must have a key name "method"'))

        for parameter, value in parameters.items():
            # Move to next iteration when parameter is 'method'
            if parameter == "method":
                continue

            # Test parameter type:
            # Str type
            if syspassclient.Config().get_data()[parameters["method"]]["params_details"][parameter]["type"].lower() == "str":

                # If parameter is not require it can be None
                if parameter in self.required_parameters(parameters["method"]):
                    if not isinstance(value, six.string_types):
                        raise TypeError("{0} must be a str instance, here that a {1}".format(parameter, type(value)))
                else:
                    if not isinstance(value, six.string_types) and value is not None:
                        raise TypeError(
                            "{0} must be a str instance or None, here that a {1}".format(parameter, type(value))
                        )

            # Int Type
            elif syspassclient.Config().get_data()[parameters["method"]]["params_details"][parameter]["type"].lower() == "int":

                # If parameter is not require it can be None
                if parameter in self.required_parameters(parameters["method"]):

                    if type(int()) != type(value):
                        raise TypeError(
                            "{0} must be a int instance, here that a {1}".format(parameter, str(type(value)))
                        )
                else:
                    if type(int()) != type(value) and value is not None:
                        raise TypeError(
                            "{0} must be a int instance or None, here that a {1}".format(parameter, str(type(value)))
                        )
            # List
            elif syspassclient.Config().get_data()[parameters["method"]]["params_details"][parameter]["type"].lower() == "array":
                # Never a array is in require parameter, and that for API v3.0 and v3.1
                if list != type(value) and value is not None:
                    raise TypeError(
                        "{0} must be a list instance or None, here that a {1}".format(parameter, str(type(value)))
                    )

    @staticmethod
    def look_for_parameters_injection(**parameters):
        """
        Internal function it look if it haven't worng parameter, only parameters define on the API can be valid.

        :param parameters: few parameters
        :type parameters: dict
        """
        if "parameters" in parameters:
            parameters = parameters["parameters"]

        if "method" not in parameters:
            raise (KeyError('"parameters" must have a key name "method"'))

        # method = parameters['method']
        # del parameters['method']

        for parameter, value in parameters.items():
            if parameter == "method":
                continue
            if parameter not in syspassclient.Config().get_data()[parameters["method"]]["params_details"]:
                raise (
                    AttributeError(
                        '"{0}" is not a valid parameter for syspass api {1}'.format(
                            parameter, syspassclient.dd.syspass["api"]["version"]
                        )
                    )
                )

    @staticmethod
    def look_for_valid_method(**parameters):
        """
        Internal function it check if a method is valid in the actual API version

        :param parameters: a keys name, keys value dictionary
        :type parameters: dict
        """

        if "parameters" in parameters:
            parameters = parameters["parameters"]

        if "method" not in parameters:
            raise (KeyError('"parameters" must have a key name "method"'))

        if parameters["method"] not in syspassclient.dd.syspass["api"][syspassclient.dd.syspass["api"]["version"]][
            "methods"]:
            raise (
                ValueError(
                    "method: {0} is not a valid on version {1} of the syspass api".format(
                        parameters["method"], syspassclient.dd.syspass["api"]["version"]
                    )
                )
            )

    @staticmethod
    def required_parameters(method):
        """
        Return a list of require item, for a special method name

        *example*

        for method='account/create'

        it should return

        ['authToken', 'categoryId', 'clientId', 'name', 'password', 'tokenPass']

        :param method: the name of the method
        :type method: str
        :return: a list of required key
        :rtype: list
        """
        requirement_list = []
        for parameter in syspassclient.Config().get_data()[method]["params_details"]:
            if syspassclient.Config().get_data()[method]["params_details"][parameter]["required"] is True:
                requirement_list.append(parameter)
        return requirement_list

    def look_for_required_params(self, **parameters):
        """
        Internal function if look if every parameters is present for a method.

        The parameter args must contain a key name 'method'

        :param parameters: parameters
        :type parameters: dict
        """
        if "parameters" in parameters:
            parameters = parameters["parameters"]

        if "method" not in parameters:
            raise (KeyError('"parameters" must have a key name "method"'))

        for required_parameter in self.required_parameters(parameters["method"]):
            if required_parameter not in parameters:
                raise (
                    AttributeError(
                        'method:{0} is require "{1}" parameter'.format(parameters["method"], required_parameter)
                    )
                )

    def merge_dictionary(self, source=None, target=None):
        """
        Merge two dicts

        :param target:
        :type target: dict
        :param source:
        :type source: dict
        :return: the merged dict
        :rtype: dict
        """
        self.is_dict_or_raise(source)
        self.is_dict_or_raise(target)

        for key, value in source.items():
            if key not in target:
                target[key] = value
            else:
                if target[key] != value:
                    target[key] = value
            # else:
            #     target[key] = merge_dictionary(target[key], value)

        return target

    @staticmethod
    def random_string(choice=string.ascii_lowercase, length=40, prefix=None):
        """
        Generate a random string of fixed length

        :param choice:
        :param length: the length of the returned string
        :type length: int
        :param prefix: use a prefix
        :type prefix: str or None
        :return: a string build with random function
        :rtype: str
        """
        if prefix is None:
            prefix = ""
        if type(prefix) != str:
            raise TypeError("'prefix' parameter must be a str type")

        return "{0}{1}".format(
            prefix,
            "".join(random.choice(choice) for _ in range(length))
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# This file is part of SysPass Client
#
# Copyright (C) 2020  DigDeo SAS
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


# import logging


class Object(object):

    def __init__(self):
        self.__verbose = True
        self.__verbose_level = 0
        self.__debug = False
        self.__debug_level = 0

        self.verbose = True
        self.verbose_level = 0
        self.debug = False
        self.debug_level = 0

        # self.logger = logging.getLogger('SyspassClient')
        # self.logger.setLevel('INFO')
        # f_handler = logging.FileHandler('/tmp/syspassclient.log')
        # f_handler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
        # # while self.logger.handlers:
        # #     self.logger.handlers.pop()
        # self.logger.addHandler(f_handler)

    @property
    def verbose(self):
        """
        Get if the verbose information's is display to the screen.

        :return: True if verbose mode is enable, False for disable it.
        :rtype: bool
        """
        return bool(self.__verbose)

    @verbose.setter
    def verbose(self, verbose):
        """
        Set if the verbose information's display on the screen.

        Generally it highly stress the console and is here for future maintenance of that Application.

        Enjoy future dev it found it function ;)

        :param verbose: True is verbose mode is enable, False for disable it.
        :type verbose: bool
        :raise TypeError: when "verbose" argument is not a :py:data:`bool`
        """
        # Exit as soon of possible
        if type(verbose) != bool:
            raise TypeError("'verbose' must be a bool type")

        # make the job in case
        if self.verbose != bool(verbose):
            self.__verbose = bool(verbose)

    @property
    def verbose_level(self):
        """
        Get the verbose information's level to display on the screen.

        Range: 0 to 3

        See: Object.set_verbose_level() for more information's about effect of ``debug_level``

        :return: The debug level as set with MorseDecoder.set_debug_level()
        :rtype: int
        """
        return self.__verbose_level

    @verbose_level.setter
    def verbose_level(self, verbose_level):
        """
        Set the verbose level of information'ss display on the screen.

        Generally it highly stress the console and is here for future maintenance of that Application.

        Enjoy future dev it found it function ;)

        :param verbose_level: The Debug level to set
        :type verbose_level: int
        :raise TypeError: when "verbose_level" argument is not a :py:data:`int`
        """
        # Exit as soon of possible
        if type(verbose_level) != int:
            raise TypeError("'verbose_level' must be a int type")

        # make the job in case
        if self.verbose_level != verbose_level:
            self.__verbose_level = verbose_level

    @property
    def debug(self):
        """
        Get the debugging information's level to display on the screen.

        :return: True if debugging mode is enable, False for disable it.
        :rtype: bool
        """
        return bool(self.__debug)

    @debug.setter
    def debug(self, debug):
        """
        Set the debugging level of information'ss display on the screen.

        Generally it highly stress the console and is here for future maintenance of that Application.

        Enjoy future dev it found it function ;)

        :param debug: True is debugging mode is enable, False for disable it.
        :type debug: bool
        :raise TypeError: when "debug" argument is not a :py:data:`bool`
        """
        # Exit as soon of possible
        if type(debug) != bool:
            raise TypeError("'debug' must be a bool type")

        # make the job in case
        if self.debug != bool(debug):
            self.__debug = bool(debug)

    @property
    def debug_level(self):
        """
        Get the debugging information's level to display on the screen.

        Range: 0 to 3

        See: MorseDecoder.set_debug_level() for more information's about effect of ``debug_level``

        :return: The debug level as set with MorseDecoder.set_debug_level()
        :rtype: int
        """
        return self.__debug_level

    @debug_level.setter
    def debug_level(self, debug_level):
        """
        Set the debugging level of information'ss display on the screen.

        Generally it highly stress the console and is here for future maintenance of that Application.

        Enjoy future dev it found it function ;)

        :param debug_level: The Debug level to set
        :type debug_level: int
        :raise TypeError: when "debug_level" argument is not a :py:data:`int`
        """
        # Exit as soon of possible
        if type(debug_level) != int:
            raise TypeError("'debug_level' must be a int type")

        # make the job in case
        if self.debug_level != debug_level:
            self.__debug_level = debug_level



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# This file is part of SysPass Client
#
# Copyright (C) 2020  DigDeo SAS
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


import os
from colorama import init, Fore, Style
import syspassclient

from yaml import load, dump

try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper
init(autoreset=True)

# Inspired by: http://zetcode.com/python/yaml/
# Inspired by: https://gitlab.com/Tuuux/galaxie-little-alice/raw/master/GLXLittleAlice/Config.py
import threading

lock = threading.Lock()


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            with lock:
                if cls not in cls._instances:
                    cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


# class Singleton(type):
#     instance = None
#
#     def __call__(cls, *args, **kw):
#         if not cls.instance:
#             cls.instance = super(Singleton, cls).__call__(*args, **kw)
#         return cls.instance


class Config(syspassclient.SyspassApi, metaclass=Singleton):
    def __init__(self):
        syspassclient.SyspassApi.__init__(self)

        # Load config file
        self.__rw = None
        self.__data = None
        self.__config_file = None
        self.__authToken = None
        self.__tokenPass = None

        self.rw = True

        self.read()

    @property
    def config_directory(self):
        if 'DD_SYSPASS_CLIENT_CONFIG_DIR' in os.environ:
            return os.path.realpath(os.environ['DD_SYSPASS_CLIENT_CONFIG_DIR'])
        else:
            return os.path.realpath(
                os.path.abspath(
                    os.path.join(
                        os.path.join(
                            os.environ['HOME'],
                            '.config'),
                        'digdeo-syspass-client'
                    )
                )
            )

    @property
    def config_file(self):
        return self.__config_file

    @config_file.setter
    def config_file(self, value=None):
        if self.config_file != value:
            self.__config_file = value

    @property
    def rw(self):
        return self.__rw

    @rw.setter
    def rw(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError('"rw" must be a bool type or None')
        if self.rw != value:
            self.__rw = value

    @property
    def authToken(self):
        # The goal is a function exit with a return as soon of possible
        # Enjoy cher!!!

        # Try the case it have more chance to match first
        if self.rw:
            # priority to the variable
            if 'DD_SYSPASS_CLIENT_AUTH_TOKEN' in os.environ:
                if self.debug and self.debug_level > 0:
                    print(Fore.WHITE + Style.BRIGHT + "DD_SYSPASS_CLIENT_AUTH_TOKEN: ", end='')
                    print(Fore.GREEN + Style.BRIGHT + "{0}".format(os.environ['DD_SYSPASS_CLIENT_AUTH_TOKEN']))
                return os.environ['DD_SYSPASS_CLIENT_AUTH_TOKEN']
            # a second chance from the config file
            if "authToken" in self.data:
                return self.data["authToken"]

        # Explicitly want a action in case of NOT , that is not a simple "else"
        if not self.rw:
            if 'DD_SYSPASS_CLIENT_AUTH_TOKEN_RO' in os.environ:
                if self.debug and self.debug_level > 0:
                    print(Fore.WHITE + Style.BRIGHT + "DD_SYSPASS_CLIENT_AUTH_TOKEN_RO: ", end='')
                    print(Fore.GREEN + Style.BRIGHT + "{0}".format(os.environ['DD_SYSPASS_CLIENT_AUTH_TOKEN_RO']))
                return os.environ['DD_SYSPASS_CLIENT_AUTH_TOKEN_RO']
            if "authTokenRO" in self.data:
                return self.data["authTokenRO"]

        # It is our else
        return None

    @property
    def tokenPass(self):
        if self.rw:
            if 'DD_SYSPASS_CLIENT_TOKEN_PASS' in os.environ:
                if self.debug and self.debug_level > 0:
                    print(Fore.WHITE + Style.BRIGHT + "DD_SYSPASS_CLIENT_TOKEN_PASS: ", end='')
                    print(Fore.GREEN + Style.BRIGHT + "{0}".format(os.environ['DD_SYSPASS_CLIENT_TOKEN_PASS']))
                return os.environ['DD_SYSPASS_CLIENT_TOKEN_PASS']
            if "tokenPass" in self.data:
                return self.data["tokenPass"]

        if not self.rw:
            if 'DD_SYSPASS_CLIENT_TOKEN_PASS_RO' in os.environ:
                if self.debug and self.debug_level > 0:
                    print(Fore.WHITE + Style.BRIGHT + "DD_SYSPASS_CLIENT_TOKEN_PASS_RO: ", end='')
                    print(Fore.GREEN + Style.BRIGHT + "{0}".format(os.environ['DD_SYSPASS_CLIENT_TOKEN_PASS_RO']))
                return os.environ['DD_SYSPASS_CLIENT_TOKEN_PASS_RO']
            if "tokenPassRO" in self.data:
                return self.data["tokenPassRO"]

        return None

    @property
    def api_url(self):
        if 'DD_SYSPASS_CLIENT_API_URL' in os.environ:
            print(Fore.WHITE + Style.BRIGHT + "DD_SYSPASS_CLIENT_API_URL: ", end='')
            print(Fore.GREEN + Style.BRIGHT + "{0}".format(os.environ['DD_SYSPASS_CLIENT_API_URL']))
            return os.environ['DD_SYSPASS_CLIENT_API_URL']
        if "api_url" in self.data:
            return self.data["api_url"]
        return None

    @property
    def data(self):
        """
        Return the config file as Python dictionary structure

        :return: Config as a big dictionary
        :rtype: dict
        """
        return self.__data

    @data.setter
    def data(self, parameters):
        """
        set en data and raise in case of error

        :param parameters: something it like a dictionary key
        :type parameters: dict
        :raise TypeError: 'parameters' is not a dict type
        """
        if self.__data != parameters:
            self.__data = parameters

    def write(self, api_file=None):
        """
        Write the config file

        :param api_file: the file where write data, or None for default one.
        :type api_file: str or None
        """
        if api_file is None:
            api_file = self.api_file

        with open(api_file, "w") as f:
            if self.debug and self.debug_level > 1:
                print(Fore.YELLOW + Style.BRIGHT + "{0}: ".format(self.__class__.__name__.upper()), end='')
                print(Fore.WHITE + Style.BRIGHT + "{0} ".format("Write API file".title()))

                print(Fore.WHITE + Style.BRIGHT + "file: ".title(), end='')
                print(str(self.api_file))
            dump(self.data, f)
            f.close()
            for line in dump(self.data).split("\n"):
                if self.verbose:
                    print(Fore.RED + Style.BRIGHT + "> " + Fore.RESET + Style.RESET_ALL + str(line))

    def read(self,
             api_file=None,
             config_file=None,
             debug=None,
             debug_level=None,
             verbose=None,
             verbose_level=None
             ):
        """
        Read the configuration file

        :param api_file: the file path to to load
        :type api_file: str or None for the default one
        :param config_file: the file it store the dd_ansible_syspass
        :type config_file: str or None for default
        :param debug: True for enable debug
        :type debug: bool
        :param debug_level: The amount of debug message . > 2 will display password in the console
        :type debug_level: int
        :param verbose: True if the syspassclient will write normal operation to the console
        :type verbose: bool
        :param verbose_level: The amount of verbose message you want >2 is really verbose
        :type verbose_level: int
        """
        if config_file is None:
            config_file = self.get_config_file()

        if api_file is None:
            api_file = self.api_file

        # with open(api_file) as f:
        #     self.data = load(f, Loader=Loader)
        #     f.close()
        #     if self.debug and self.debug_level > 1:
        #         print(Fore.YELLOW + Style.BRIGHT + "{0}: ".format(self.__class__.__name__.upper()), end='')
        #         print(Fore.WHITE + Style.BRIGHT + "{0} ".format("Load API File"))
        #
        #         print(Fore.WHITE + Style.BRIGHT + "File: ", end='')
        #         print(api_file)
        #
        #         if self.debug and self.debug_level > 2:
        #             for line in dump(self.data).split("\n"):
        #                 if self.verbose:
        #                     print(Fore.CYAN + Style.BRIGHT + "< " + Fore.RESET + Style.RESET_ALL + str(line))
        self.read_api()
        self.data = self.api_data

        with open(config_file) as f:
            config = load(f, Loader=Loader)
            f.close()
            # print(config)

            if 'syspassclient' not in config:
                raise AttributeError('{0} do not contain syspassclient attribute'.format(config_file))

            if config["syspassclient"] is None:
                raise ImportError('nothing to import ...')

            if "api_url" in config["syspassclient"]:
                self.data["api_url"] = config["syspassclient"]["api_url"]

            if "api_version" in config["syspassclient"]:
                self.data["api_version"] = config["syspassclient"]["api_version"]
                self.api_version = self.data["api_version"]

            if "authToken" in config["syspassclient"]:
                self.data["authToken"] = config["syspassclient"]["authToken"]

            if "tokenPass" in config["syspassclient"]:
                self.data["tokenPass"] = config["syspassclient"]["tokenPass"]

            if "debug" in config["syspassclient"]:
                self.data["debug"] = config["syspassclient"]["debug"]

            if "debug_level" in config["syspassclient"]:
                self.data["debug_level"] = config["syspassclient"]["debug_level"]

            if "verbose" in config["syspassclient"]:
                self.data["verbose"] = config["syspassclient"]["verbose"]

            if "verbose_level" in config["syspassclient"]:
                self.data["verbose_level"] = config["syspassclient"]["verbose_level"]

            if debug is None:
                if "debug" in self.data:
                    self.debug = self.data["debug"]
                else:
                    self.debug = True
            else:
                self.debug = debug

            if debug_level is None:
                if "debug_level" in self.data:
                    self.debug_level = self.data["debug_level"]
                else:
                    self.debug_level = 3
            else:
                self.debug_level = debug_level

            if verbose is None:
                if "verbose" in self.data:
                    self.verbose = self.data["verbose"]
                else:
                    self.verbose = True
            else:
                self.verbose = verbose

            if verbose_level is None:
                if "verbose_level" in self.data:
                    self.verbose_level = self.data["verbose_level"]
                else:
                    self.verbose_level = 3
            else:
                self.verbose_level = verbose_level

        if self.debug and self.debug_level > 1:
            print(Fore.YELLOW + Style.BRIGHT + "{0}: ".format(self.__class__.__name__.upper()), end='')
            print(Fore.WHITE + Style.BRIGHT + "{0} ".format("Load Config File"))

            print(Fore.WHITE + Style.BRIGHT + "File: ", end='')
            print(self.get_config_file())

            if self.debug_level > 2:
                print(
                    Fore.WHITE + Style.BRIGHT +
                    "api_url:" +
                    Style.RESET_ALL +
                    " {0}".format(self.data["api_url"])
                )
                print(
                    Fore.WHITE + Style.BRIGHT +
                    "authToken:" +
                    Style.RESET_ALL +
                    " {0}".format(self.authToken)
                )
                print(
                    Fore.WHITE + Style.BRIGHT +
                    "tokenPass:" +
                    Style.RESET_ALL +
                    " {0}".format(self.tokenPass)
                )

            print(
                Fore.WHITE + Style.BRIGHT +
                "debug:" +
                Style.RESET_ALL + Fore.RESET +
                " {0}".format(self.debug)
            )
            print(
                Fore.WHITE + Style.BRIGHT +
                "debug_level:" +
                Style.RESET_ALL + Fore.RESET +
                " {0}".format(self.debug_level)
            )
            print(
                Fore.WHITE + Style.BRIGHT +
                "verbose:" +
                Style.RESET_ALL + Fore.RESET +
                " {0}".format(self.verbose)
            )
            print(
                Fore.WHITE + Style.BRIGHT + "verbose_level:" + Style.RESET_ALL + Fore.RESET +
                " {0}".format(self.verbose_level)
            )

    def reset(self):
        if self.api_version == "3.0":
            self.data = self.api_3_0
        else:
            self.data = self.api_3_1

    def get_data(self):
        """
        return teh data dictionary

        :return: the data dictionary
        :rtype: dit
        """
        return self.data

    @staticmethod
    def get_empty_config_dict():
        return {
            'syspassclient': {
                'api_url': None,
                'api_version': '3.1',
                'authToken': None,
                'tokenPass': None,
                'debug': True,
                'debug_level': 3,
                'verbose': False,
                'verbose_level': 0
            }
        }

    def get_config_file(self):
        return os.path.join(
            self.config_directory,
            "config.yml",
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-


# This file is part of SysPass Client
#
# Copyright (C) 2020  DigDeo SAS
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

import re
import syspassclient


class CheckType(object):
    def __init__(self):
        pass

    @staticmethod
    def is_str_or_raise(obj=None):
        if not isinstance(obj, str):
            raise TypeError("{0} must be a str type".format(str(obj)))

    @staticmethod
    def is_int_or_raise(obj=None):
        if not isinstance(obj, int):
            raise TypeError("{0} must be a int type".format(str(obj)))

    @staticmethod
    def is_float_or_raise(obj=None):
        if not isinstance(obj, float):
            raise TypeError("{0} must be a float type".format(str(obj)))

    @staticmethod
    def is_dict_or_raise(obj=None):
        if not isinstance(obj, dict):
            raise TypeError("{0} must be a dict type".format(str(obj)))

    @staticmethod
    def is_list_or_raise(obj=None):
        if not isinstance(obj, list):
            raise TypeError("{0} must be a list type".format(str(obj)))

    @staticmethod
    def is_bool_or_raise(obj=None):
        if not isinstance(obj, bool):
            raise TypeError("{0} must be a bool type".format(str(obj)))

    def is_url_or_raise(self, string_url=None):
        """
        Test if a string url is a valide url

        Note if ``string_url`` is set to None the function return FAlse, that because None is not a valide URL.
        :param string_url: a url to check
        :type string_url: str or None
        :return: True if ``string`` contain only valid ASCII character or False if not
        :rtype: bool
        """
        self.is_str_or_raise(string_url)

        regex = re.compile(
            r'^(?:http|ftp)s?://'  # http:// or https://
            r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
            r'localhost|'  # localhost...
            r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
            r'(?::\d+)?'  # optional port
            r'(?:/?|[/?]\S+)$', re.IGNORECASE)

        if re.match(regex, string_url) is None:
            raise ValueError("'api_url' must be a valid URL")

    def is_ascii_or_raise(self, string_data=None):
        """
        Test if a string contain only ASCCI char.

        If ``string_data`` is set to None , the function just raise a TypeError, because None is not a ASCII char.

        :param string_data: the string to check
        :type string_data: str
        :raise TypeError: when string_data is not a str
        :raise ValueError: when string_data contain a none ascii char
        """
        self.is_str_or_raise(string_data)

        for character in string_data:
            if character not in syspassclient.dd.ascii_all:
                raise ValueError("{0} must be contain only ascii char".format(str(string_data)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of SysPass Client
#
# Copyright (C) 2020  DigDeo SAS
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
__all__ = [
    'Constants',
    'dd',
    'Object',
    'Config',
    'config',
    'CheckType',
    'Utils',
    'SyspassClient'
]

from syspassclient.constants import Constants
from syspassclient.constants import dd
from syspassclient.object import Object
from syspassclient.api import SyspassApi
from syspassclient.config import Config
from syspassclient.check_type import CheckType
from syspassclient.utils import Utils
from syspassclient.syspassclient import SyspassClient

# require for digdeo-syspass-ansible-lookup
config = Config()



#!/usr/bin/env python

# This will try to import setuptools. If not here, it fails with a message
import os

try:
    from setuptools import setup, find_packages
except ImportError:
    raise ImportError(
        "This module could not be installed, probably because"
        " setuptools is not installed on this computer."
        "\nInstall ez_setup ([sudo] pip install ez_setup) and try again."
    )

pre_version = "0.4.6rc1"
if os.environ.get('CI_COMMIT_TAG'):
    version = os.environ['CI_COMMIT_TAG']
else:
    if os.environ.get('CI_JOB_ID'):
        version = os.environ['CI_JOB_ID']
    else:
        version = pre_version

with open('README.md') as f:
    long_description = f.read()

setup(
    name='digdeo-syspass-client',
    version=version,
    description='DigDeo Syspass Client',
    author='DigDeo',
    author_email='jerome.ornech@digdeo.fr',
    include_package_data=True,
    url='https://git.digdeo.fr/digdeo-system/FLOSS/digdeo-syspass-client/',
    license='GNU GENERAL PUBLIC LICENSE Version 3',
    zip_safe=False,
    long_description=long_description,
    long_description_content_type='text/markdown; charset=UTF-8',
    keywords="DigDeo Syspass Client",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Security",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5"
    ],
    setup_requires=[
        'green',
        'wheel'
    ],
    packages=['syspassclient'],
    tests_require=[
        'wheel',
        'colorama',
        'six',
        'PyYAML',
        'requests',
        'urllib3'
    ],
    install_requires=[
        'colorama',
        'six',
        'PyYAML',
        'requests',
        'urllib3'
    ]
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# This file is part of SysPass Client
#
# Copyright (C) 2020  DigDeo SAS
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
import os
from colorama import init, Fore, Style
import syspassclient

from yaml import load, dump

try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper
init(autoreset=True)


class SyspassApi(syspassclient.Object):
    def __init__(self):
        syspassclient.Object.__init__(self)
        self.__api_directory = None
        self.__api_filename = None
        self.__api_filename_ext = None
        self.__api_file = None
        self.__api_version = None
        self.__data = None

        self.api_directory = None
        self.api_filename_ext = None

    @property
    def api_data(self):
        """
        Return the config file as Python dictionary structure

        :return: Config as a big dictionary
        :rtype: dict
        """
        return self.__data

    @api_data.setter
    def api_data(self, parameters):
        """
        set en data and raise in case of error

        :param parameters: something it like a dictionary key
        :type parameters: dict
        :raise TypeError: 'parameters' is not a dict type
        """
        if self.__data != parameters:
            self.__data = parameters

    @property
    def api_3_0(self):
        return {
            "account/search": {
                # Search for accounts
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # text 	        string 	no 	        Text to search for
                # count 	    int 	no 	        Number of results to display
                # categoryId 	int 	no 	        Category’s Id for filtering
                # clientId 	    int 	no 	        Client’s Id for filtering
                # tagsId 	    array 	no 	        Tags’ Id for filtering
                # op 	        string 	no 	        Operator used for filtering. It can be either ‘or’ or ‘and’
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "text": {"type": "str", "required": False},
                    "count": {"type": "int", "required": False},
                    "categoryId": {"type": "int", "required": False},
                    "clientId": {"type": "int", "required": False},
                    "tagsId": {"type": "array", "required": False},
                    "op": {"type": "str", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "account/search",
                    "params": {
                        "authToken": None,
                        "text": None,
                        "count": None,
                        "categoryId": None,
                        "clientId": None,
                        "tagsid": None,
                        "op": None,
                    },
                    "id": None,
                },
            },
            "account/view": {
                # Get account’s details
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # tokenPass 	string 	yes 	    API token’s pass
                # id 	        int 	yes 	    Account’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tokenPass": {"type": "str", "required": True},
                    "account_id": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "account/view",
                    "params": {"authToken": None, "account_id": None, "tokenPass": None},
                    "id": None,
                },
            },
            "account/viewPass": {
                # Get account’s password
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # tokenPass 	string 	yes 	    API token’s pass
                # id 	        int 	yes 	    Account’s Id
                # details 	    int 	no 	        Whether to return account’s details within response account/editPass
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tokenPass": {"type": "str", "required": True},
                    "account_id": {"type": "int", "required": True},
                    "details": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "account/viewPass",
                    "params": {"authToken": None, "account_id": None, "tokenPass": None, "details": None},
                    "id": None,
                },
            },
            "account/editPass": {
                # Edit account’s password
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	User’s API token
                # tokenPass 	string 	yes 	API token’s pass
                # id 	int 	yes 	Account’s Id
                # pass 	string 	yes 	Account’s password
                # expireDate 	int 	no 	Expire date in UNIX timestamp format
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tokenPass": {"type": "str", "required": True},
                    "account_id": {"type": "int", "required": True},
                    "password": {"type": "str", "required": True},
                    "expireDate": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "account/editPass",
                    "params": {"authToken": None, "tokenPass": None, "account_id": None, "password": None,
                               "expireDate": None},
                    "id": None,
                },
            },
            "account/create": {
                # Create account
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	User’s API token
                # tokenPass 	string 	yes 	API token’s pass
                # name 	        string 	yes 	Account’s name
                # categoryId 	int 	yes 	Account’s category Id
                # clientId 	    int 	yes 	Account’s client Id
                # pass 	        string 	yes 	Account’s password
                # tagsId 	    array 	no 	    Account’s tags Id
                # userGroupId 	int 	no 	    Account’s user group Id
                # parentId 	    int 	no 	    Account’s parent Id
                # login 	    string 	no 	    Account’s login
                # url 	        string 	no 	    Account’s access URL or IP
                # notes 	    string 	no 	    Account’s notes
                # private 	    int 	no 	    Set account as private. It can be either 0 or 1
                # privateGroup 	int 	no 	    Set account as private for group. It can be either 0 or 1
                # expireDate 	int 	no 	    Expire date in UNIX timestamp format
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tokenPass": {"type": "str", "required": True},
                    "name": {"type": "str", "required": True},
                    "categoryId": {"type": "int", "required": True},
                    "clientId": {"type": "int", "required": True},
                    "password": {"type": "str", "required": True},
                    "tagsId": {"type": "array", "required": False},
                    "userGroupId": {"type": "int", "required": False},
                    "parentId": {"type": "int", "required": False},
                    "login": {"type": "str", "required": False},
                    "url": {"type": "str", "required": False},
                    "notes": {"type": "str", "required": False},
                    "private": {"type": "int", "required": False},
                    "privateGroup": {"type": "int", "required": False},
                    "expireDate": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "account/create",
                    "params": {
                        "authToken": None,
                        "tokenPass": None,
                        "name": None,
                        "categoryId": None,
                        "clientId": None,
                        "password": None,
                        "tagsId": None,
                        "userGroupId": None,
                        "parentId": None,
                        "login": None,
                        "url": None,
                        "notes": None,
                        "private": None,
                        "privateGroup": None,
                        "expireDate": None,
                    },
                    "id": None,
                },
            },
            "account/edit": {
                # Create account
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	User’s API token
                # tokenPass 	string 	yes 	API token’s pass
                # id 	        int 	yes 	Account’s Id
                # name 	        string 	no 	    Account’s name
                # categoryId 	int 	no 	    Account’s category Id
                # clientId 	    int 	no 	    Account’s client Id
                # tagsId 	    array 	no 	    Account’s tags Id
                # userGroupId 	int 	no 	    Account’s user group Id
                # parentId 	    int 	no 	    Account’s parent Id
                # login 	    string 	no 	    Account’s login
                # url 	        string 	no 	    Account’s access URL or IP
                # notes 	    string 	no 	    Account’s notes
                # private 	    int 	no 	    Set account as private. It can be either 0 or 1
                # privateGroup 	int 	no 	    Set account as private for group. It can be either 0 or 1
                # expireDate 	int 	no 	    Expire date in UNIX timestamp format
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tokenPass": {"type": "str", "required": True},
                    "account_id": {"type": "int", "required": True},
                    "name": {"type": "str", "required": False},
                    "categoryId": {"type": "int", "required": False},
                    "clientId": {"type": "int", "required": False},
                    "tagsId": {"type": "array", "required": False},
                    "userGroupId": {"type": "int", "required": False},
                    "parentId": {"type": "int", "required": False},
                    "login": {"type": "str", "required": False},
                    "url": {"type": "str", "required": False},
                    "notes": {"type": "str", "required": False},
                    "private": {"type": "int", "required": False},
                    "privateGroup": {"type": "int", "required": False},
                    "expireDate": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "account/edit",
                    "params": {
                        "authToken": None,
                        "tokenPass": None,
                        "account_id": None,
                        "name": None,
                        "categoryId": None,
                        "clientId": None,
                        "tagsId": None,
                        "userGroupId": None,
                        "parentId": None,
                        "login": None,
                        "url": None,
                        "notes": None,
                        "private": None,
                        "privateGroup": None,
                        "expireDate": None,
                    },
                    "id": None,
                },
            },
            "account/delete": {
                # Delete an account
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	User’s API token
                # id 	        int 	yes 	Account’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "account_id": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "account/delete",
                    "params": {"authToken": None, "account_id": None},
                    "id": None,
                },
            },
            "category/search": {
                # Search for categories
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # text 	        string 	no 	        Text to search for
                # count 	    int 	no 	        Number of results to display
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "text": {"type": "str", "required": False},
                    "count": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "category/search",
                    "params": {"authToken": None, "text": None, "count": None},
                    "id": None,
                },
            },
            "category/view": {
                # Get category’s details
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # text 	        string 	no 	        Text to search for
                # count 	    int 	no 	        Number of results to display
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "text": {"type": "str", "required": True},
                    "count": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "category/view",
                    "params": {"authToken": None, "text": None, "count": None},
                    "id": None,
                },
            },
            "category/create": {
                # Create category
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # name 	        string 	yes 	    Category’s name
                # description 	string 	no 	        Category’s description
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "name": {"type": "str", "required": True},
                    "description": {"type": "str", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "category/create",
                    "params": {"authToken": None, "name": None, "description": None},
                    "id": None,
                },
            },
            "category/edit": {
                # Edit category
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # id 	        int 	yes 	    Category’s Id
                # name 	        string 	yes 	    Category’s name
                # description 	string 	no 	        Category’s description
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "cid": {"type": "int", "required": True},
                    "name": {"type": "str", "required": True},
                    "description": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "category/edit",
                    "params": {"authToken": None, "cid": None, "name": None, "description": None},
                    "id": None,
                },
            },
            "category/delete": {
                # Delete category
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # id 	        int 	yes 	    Category’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "cid": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "category/delete",
                    "params": {"authToken": None, "cid": None},
                    "cid": None,
                },
            },
            "client/search": {
                # Search for clients
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # text 	        string 	no 	        Text to search for
                # count 	    int 	no 	        Number of results to display
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "text": {"type": "str", "required": False},
                    "count": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "client/search",
                    "params": {"authToken": None, "text": None, "count": None},
                    "id": None,
                },
            },
            "client/view": {
                # Get client’s details
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # tokenPass 	string 	yes 	    API token’s pass
                # id 	        int 	yes 	    Client’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tokenPass": {"type": "str", "required": True},
                    "cid": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "client/view",
                    "params": {"authToken": None, "tokenPass": None, "cid": None},
                    "id": None,
                },
            },
            "client/create": {
                # Get client’s details
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	User’s API token
                # name 	        string 	yes 	Client’s name
                # description 	string 	no 	    Client’s description
                # Global 	    int 	no 	    Set client as Global. It can be either 0 or 1
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "name": {"type": "str", "required": True},
                    "description": {"type": "str", "required": False},
                    "Global": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "client/create",
                    "params": {"authToken": None, "name": None, "description": None, "Global": None},
                    "id": None,
                },
            },
            "client/edit": {
                # Edit client
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # id 	        int 	yes 	    Client’s Id
                # name 	        string 	yes 	    Client’s name
                # description 	string 	no 	        Client’s description
                # Global 	    int 	no 	        Set client as Global. It can be either 0 or 1
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "cid": {"type": "str", "required": True},
                    "name": {"type": "str", "required": True},
                    "description": {"type": "int", "required": False},
                    "Global": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "client/edit",
                    "params": {"authToken": None, "cid": None, "name": None, "description": None, "Global": None},
                    "id": None,
                },
            },
            "client/delete": {
                # Delete client
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # id 	        int 	yes 	    Client’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "cid": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "client/delete",
                    "params": {"authToken": None, "cid": None},
                    "id": None,
                },
            },
            "tag/search": {
                # Search for tags
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # text 	        string 	no 	        Text to search for
                # count 	    int 	no 	        Number of results to display
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "text": {"type": "str", "required": False},
                    "count": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "tag/search",
                    "params": {"authToken": None, "text": None, "count": None},
                    "id": None,
                },
            },
            "tag/view": {
                # Get tag’s details
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # tokenPass 	string 	yes 	    API token’s pass
                # id 	        int 	yes 	    Tag’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tokenPass": {"type": "str", "required": True},
                    "tagid": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "tag/view",
                    "params": {"authToken": None, "tokenPass": None, "tagid": None},
                    "id": None,
                },
            },
            "tag/create": {
                # Create tag
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # name 	        string 	yes 	    API token’s pass
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "name": {"type": "str", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "tag/create",
                    "params": {"authToken": None, "name": None},
                    "id": None,
                },
            },
            "tag/edit": {
                # Create tag
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # id 	        int 	yes 	    Tag’s Id
                # name 	        string 	yes 	    Tag’s name
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "id": {"type": "int", "required": True},
                    "name": {"type": "str", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "tag/edit",
                    "params": {"authToken": None, "id": None, "name": None},
                    "id": None,
                },
            },
            "tag/delete": {
                # Delete tag
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # id 	        int 	yes 	    Tag’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tagid": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "tag/delete",
                    "params": {"authToken": None, "tagid": None},
                    "id": None,
                },
            },
            "usergroup/search": {
                # Search for user groups
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # text 	        string 	no 	        Text to search for
                # count 	    int 	no 	        Number of results to display
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "text": {"type": "str", "required": False},
                    "count": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "usergroup/search",
                    "params": {"authToken": None, "text": None, "count": None},
                    "id": None,
                },
            },
            "usergroup/view": {
                # Get user group’s details
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # tokenPass 	string 	yes 	    API token’s pass
                # id 	        int 	yes 	    User group’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tokenPass": {"type": "str", "required": True},
                    "ugid": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "usergroup/view",
                    "params": {"authToken": None, "tokenPass": None, "ugid": None},
                    "id": None,
                },
            },
            "usergroup/create": {
                # Create user group
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # name 	        string 	yes 	    User group’s name
                # description 	string 	no 	        User group’s description
                # usersId 	    array 	no 	        User group’s users Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "name": {"type": "str", "required": True},
                    "description": {"type": "str", "required": False},
                    "usersId": {"type": "array", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "usergroup/create",
                    "params": {"authToken": None, "name": None, "description": None, "usersId": None},
                    "id": None,
                },
            },
            "usergroup/edit": {
                # Edit user group
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # id 	        int 	yes 	    User group’s Id
                # name 	        string 	yes 	    User group’s name
                # description 	string 	no 	        User group’s description
                # usersId 	    array 	no 	        User group’s users Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "ugid": {"type": "int", "required": True},
                    "name": {"type": "string", "required": True},
                    "description": {"type": "string", "required": False},
                    "usersId": {"type": "array", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "usergroup/edit",
                    "params": {"authToken": None, "ugid": None, "name": None, "description": None, "usersId": None},
                    "id": None,
                },
            },
            "usergroup/delete": {
                # Delete user group
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # id 	        int 	yes 	    User group’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "ugid": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "usergroup/delete",
                    "params": {"authToken": None, "ugid": None},
                    "ugid": None,
                },
            },
            "config/backup": {
                # Perform an application and database backup
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # path 	        string 	no 	        Server path to store the application and database backup
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "path": {"type": "str", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "config/backup",
                    "params": {"authToken": None, "path": None},
                    "id": None,
                },
            },
            "config/export": {
                # Export application data in XML format
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # path 	        string 	no 	        Server path to store the XML file
                # password 	    string 	no 	        Password used to encrypt the exported data
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "path": {"type": "str", "required": False},
                    "password": {"type": "str", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "config/export",
                    "params": {"authToken": None, "path": None, "password": None},
                    "id": None,
                },
            },
        }

    @property
    def api_3_1(self):
        return {
            "account/search": {
                # Search for accounts
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # text 	        string 	no 	        Text to search for
                # count 	    int 	no 	        Number of results to display
                # categoryId 	int 	no 	        Category’s Id for filtering
                # clientId 	    int 	no 	        Client’s Id for filtering
                # tagsId 	    array 	no 	        Tags’ Id for filtering
                # op 	        string 	no 	        Operator used for filtering. It can be either ‘or’ or ‘and’
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "text": {"type": "str", "required": False},
                    "count": {"type": "int", "required": False},
                    "categoryId": {"type": "int", "required": False},
                    "clientId": {"type": "int", "required": False},
                    "tagsId": {"type": "array", "required": False},
                    "op": {"type": "str", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "account/search",
                    "params": {
                        "authToken": None,
                        "text": None,
                        "count": None,
                        "categoryId": None,
                        "clientId": None,
                        "tagsid": None,
                        "op": None,
                    },
                    "id": None,
                },
            },
            "account/view": {
                # Get account’s details
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # tokenPass 	string 	yes 	    API token’s pass
                # id 	        int 	yes 	    Account’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tokenPass": {"type": "str", "required": True},
                    "account_id": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "account/view",
                    "params": {"authToken": None, "account_id": None, "tokenPass": None},
                    "id": None,
                },
            },
            "account/viewPass": {
                # Get account’s password
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # tokenPass 	string 	yes 	    API token’s pass
                # id 	        int 	yes 	    Account’s Id
                # details 	    int 	no 	        Whether to return account’s details within response account/editPass
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tokenPass": {"type": "str", "required": True},
                    "account_id": {"type": "int", "required": True},
                    "details": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "account/viewPass",
                    "params": {"authToken": None, "account_id": None, "tokenPass": None, "details": None},
                    "id": None,
                },
            },
            "account/editPass": {
                # Edit account’s password
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	User’s API token
                # tokenPass 	string 	yes 	API token’s pass
                # id 	int 	yes 	Account’s Id
                # pass 	string 	yes 	Account’s password
                # expireDate 	int 	no 	Expire date in UNIX timestamp format
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tokenPass": {"type": "str", "required": True},
                    "account_id": {"type": "int", "required": True},
                    "password": {"type": "str", "required": True},
                    "expireDate": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "account/editPass",
                    "params": {"authToken": None,
                               "tokenPass": None,
                               "account_id": None,
                               "password": None,
                               "expireDate": None
                               },
                    "id": None,
                },
            },
            "account/create": {
                # Create account
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	User’s API token
                # tokenPass 	string 	yes 	API token’s pass
                # name 	        string 	yes 	Account’s name
                # categoryId 	int 	yes 	Account’s category Id
                # clientId 	    int 	yes 	Account’s client Id
                # pass 	        string 	yes 	Account’s password
                # tagsId 	    array 	no 	    Account’s tags Id
                # userGroupId 	int 	no 	    Account’s user group Id
                # parentId 	    int 	no 	    Account’s parent Id
                # login 	    string 	no 	    Account’s login
                # url 	        string 	no 	    Account’s access URL or IP
                # notes 	    string 	no 	    Account’s notes
                # private 	    int 	no 	    Set account as private. It can be either 0 or 1
                # privateGroup 	int 	no 	    Set account as private for group. It can be either 0 or 1
                # expireDate 	int 	no 	    Expire date in UNIX timestamp format
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tokenPass": {"type": "str", "required": True},
                    "name": {"type": "str", "required": True},
                    "categoryId": {"type": "int", "required": True},
                    "clientId": {"type": "int", "required": True},
                    "password": {"type": "str", "required": True},
                    "tagsId": {"type": "array", "required": False},
                    "userGroupId": {"type": "int", "required": False},
                    "parentId": {"type": "int", "required": False},
                    "login": {"type": "str", "required": False},
                    "url": {"type": "str", "required": False},
                    "notes": {"type": "str", "required": False},
                    "private": {"type": "int", "required": False},
                    "privateGroup": {"type": "int", "required": False},
                    "expireDate": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "account/create",
                    "params": {
                        "authToken": None,
                        "tokenPass": None,
                        "name": None,
                        "categoryId": None,
                        "clientId": None,
                        "password": None,
                        "tagsId": None,
                        "userGroupId": None,
                        "parentId": None,
                        "login": None,
                        "url": None,
                        "notes": None,
                        "private": None,
                        "privateGroup": None,
                        "expireDate": None,
                    },
                    "id": None,
                },
            },
            "account/edit": {
                # Create account
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	User’s API token
                # tokenPass 	string 	yes 	API token’s pass
                # id 	        int 	yes 	Account’s Id
                # name 	        string 	no 	    Account’s name
                # categoryId 	int 	no 	    Account’s category Id
                # clientId 	    int 	no 	    Account’s client Id
                # tagsId 	    array 	no 	    Account’s tags Id
                # userGroupId 	int 	no 	    Account’s user group Id
                # parentId 	    int 	no 	    Account’s parent Id
                # login 	    string 	no 	    Account’s login
                # url 	        string 	no 	    Account’s access URL or IP
                # notes 	    string 	no 	    Account’s notes
                # private 	    int 	no 	    Set account as private. It can be either 0 or 1
                # privateGroup 	int 	no 	    Set account as private for group. It can be either 0 or 1
                # expireDate 	int 	no 	    Expire date in UNIX timestamp format
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tokenPass": {"type": "str", "required": True},
                    "account_id": {"type": "int", "required": True},
                    "name": {"type": "str", "required": False},
                    "categoryId": {"type": "int", "required": False},
                    "clientId": {"type": "int", "required": False},
                    "tagsId": {"type": "array", "required": False},
                    "userGroupId": {"type": "int", "required": False},
                    "parentId": {"type": "int", "required": False},
                    "login": {"type": "str", "required": False},
                    "url": {"type": "str", "required": False},
                    "notes": {"type": "str", "required": False},
                    "private": {"type": "int", "required": False},
                    "privateGroup": {"type": "int", "required": False},
                    "expireDate": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "account/edit",
                    "params": {
                        "authToken": None,
                        "tokenPass": None,
                        "account_id": None,
                        "name": None,
                        "categoryId": None,
                        "clientId": None,
                        "tagsId": None,
                        "userGroupId": None,
                        "parentId": None,
                        "login": None,
                        "url": None,
                        "notes": None,
                        "private": None,
                        "privateGroup": None,
                        "expireDate": None,
                    },
                    "id": None,
                },
            },
            "account/delete": {
                # Delete an account
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	User’s API token
                # id 	        int 	yes 	Account’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "account_id": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "account/delete",
                    "params": {"authToken": None, "account_id": None},
                    "id": None,
                },
            },
            "category/search": {
                # Search for categories
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # text 	        string 	no 	        Text to search for
                # count 	    int 	no 	        Number of results to display
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "text": {"type": "str", "required": False},
                    "count": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "category/search",
                    "params": {"authToken": None, "text": None, "count": None},
                    "id": None,
                },
            },
            "category/view": {
                # Get category’s details
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # text 	        string 	no 	        Text to search for
                # count 	    int 	no 	        Number of results to display
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "text": {"type": "str", "required": True},
                    "count": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "category/view",
                    "params": {"authToken": None, "text": None, "count": None},
                    "id": None,
                },
            },
            "category/create": {
                # Create category
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # name 	        string 	yes 	    Category’s name
                # description 	string 	no 	        Category’s description
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "name": {"type": "str", "required": True},
                    "description": {"type": "str", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "category/create",
                    "params": {"authToken": None, "name": None, "description": None},
                    "id": None,
                },
            },
            "category/edit": {
                # Edit category
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # id 	        int 	yes 	    Category’s Id
                # name 	        string 	yes 	    Category’s name
                # description 	string 	no 	        Category’s description
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "cid": {"type": "int", "required": True},
                    "name": {"type": "str", "required": True},
                    "description": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "category/edit",
                    "params": {"authToken": None, "cid": None, "name": None, "description": None},
                    "id": None,
                },
            },
            "category/delete": {
                # Delete category
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # id 	        int 	yes 	    Category’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "cid": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "category/delete",
                    "params": {"authToken": None, "id": None},
                    "cid": None,
                },
            },
            "client/search": {
                # Search for clients
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # text 	        string 	no 	        Text to search for
                # count 	    int 	no 	        Number of results to display
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "text": {"type": "str", "required": False},
                    "count": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "client/search",
                    "params": {"authToken": None, "text": None, "count": None},
                    "id": None,
                },
            },
            "client/view": {
                # Get client’s details
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # tokenPass 	string 	yes 	    API token’s pass
                # id 	        int 	yes 	    Client’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tokenPass": {"type": "str", "required": True},
                    "cid": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "client/view",
                    "params": {"authToken": None, "tokenPass": None, "cid": None},
                    "id": None,
                },
            },
            "client/create": {
                # Get client’s details
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	User’s API token
                # name 	        string 	yes 	Client’s name
                # description 	string 	no 	    Client’s description
                # Global 	    int 	no 	    Set client as Global. It can be either 0 or 1
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "name": {"type": "str", "required": True},
                    "description": {"type": "str", "required": False},
                    "Global": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "client/create",
                    "params": {"authToken": None, "name": None, "description": None, "Global": None},
                    "id": None,
                },
            },
            "client/edit": {
                # Edit client
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # id 	        int 	yes 	    Client’s Id
                # name 	        string 	yes 	    Client’s name
                # description 	string 	no 	        Client’s description
                # Global 	    int 	no 	        Set client as Global. It can be either 0 or 1
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "cid": {"type": "str", "required": True},
                    "name": {"type": "str", "required": True},
                    "description": {"type": "int", "required": False},
                    "Global": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "client/edit",
                    "params": {"authToken": None, "cid": None, "name": None, "description": None, "Global": None},
                    "id": None,
                },
            },
            "client/delete": {
                # Delete client
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # cid 	        int 	yes 	    Client’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "cid": {"type": "int", "required": True}
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "client/delete",
                    "params": {"authToken": None, "cid": None},
                    "id": None,
                },
            },
            "tag/search": {
                # Search for tags
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # text 	        string 	no 	        Text to search for
                # count 	    int 	no 	        Number of results to display
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "text": {"type": "str", "required": False},
                    "count": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "tag/search",
                    "params": {"authToken": None, "text": None, "count": None},
                    "id": None,
                },
            },
            "tag/view": {
                # Get tag’s details
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # tokenPass 	string 	yes 	    API token’s pass
                # tagid 	    int 	yes 	    Tag’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tokenPass": {"type": "str", "required": True},
                    "tagid": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "tag/view",
                    "params": {"authToken": None, "tokenPass": None, "tagid": None},
                    "id": None,
                },
            },
            "tag/create": {
                # Create tag
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # name 	        string 	yes 	    API token’s pass
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "name": {"type": "str", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "tag/create",
                    "params": {"authToken": None, "name": None},
                    "id": None,
                },
            },
            "tag/edit": {
                # Create tag
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # tagid 	    int 	yes 	    Tag’s Id
                # name 	        string 	yes 	    Tag’s name
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tagid": {"type": "int", "required": True},
                    "name": {"type": "str", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "tag/edit",
                    "params": {"authToken": None, "id": None, "name": None},
                    "id": None,
                },
            },
            "tag/delete": {
                # Delete tag
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # tagid 	    int 	yes 	    Tag’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tagid": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "tag/delete",
                    "params": {"authToken": None, "tagid": None},
                    "id": None,
                },
            },
            "userGroup/search": {
                # Search for user groups
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # text 	        string 	no 	        Text to search for
                # count 	    int 	no 	        Number of results to display
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "text": {"type": "str", "required": False},
                    "count": {"type": "int", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "userGroup/search",
                    "params": {"authToken": None, "text": None, "count": None},
                    "id": None,
                },
            },
            "userGroup/view": {
                # Get user group’s details
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # tokenPass 	string 	yes 	    API token’s pass
                # id 	        int 	yes 	    User group’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "tokenPass": {"type": "str", "required": True},
                    "ugid": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "usergroup/view",
                    "params": {"authToken": None, "tokenPass": None, "ugid": None},
                    "id": None,
                },
            },
            "userGroup/create": {
                # Create user group
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # name 	        string 	yes 	    User group’s name
                # description 	string 	no 	        User group’s description
                # usersId 	    array 	no 	        User group’s users Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "name": {"type": "str", "required": True},
                    "description": {"type": "str", "required": False},
                    "usersId": {"type": "array", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "usergroup/create",
                    "params": {"authToken": None, "name": None, "description": None, "usersId": None},
                    "id": None,
                },
            },
            "userGroup/edit": {
                # Edit user group
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # id 	        int 	yes 	    User group’s Id
                # name 	        string 	yes 	    User group’s name
                # description 	string 	no 	        User group’s description
                # usersId 	    array 	no 	        User group’s users Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "ugid": {"type": "int", "required": True},
                    "name": {"type": "string", "required": True},
                    "description": {"type": "string", "required": False},
                    "usersId": {"type": "array", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "usergroup/edit",
                    "params": {"authToken": None, "ugid": None, "name": None, "description": None, "usersId": None},
                    "id": None,
                },
            },
            "userGroup/delete": {
                # Delete user group
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # id 	        int 	yes 	    User group’s Id
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "ugid": {"type": "int", "required": True},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "usergroup/delete",
                    "params": {"authToken": None, "ugid": None},
                    "ugid": None,
                },
            },
            "config/backup": {
                # Perform an application and database backup
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # path 	        string 	no 	        Server path to store the application and database backup
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "path": {"type": "str", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "config/backup",
                    "params": {"authToken": None, "path": None},
                    "id": None,
                },
            },
            "config/export": {
                # Export application data in XML format
                # Parameter 	Type 	Required 	Description
                # authToken 	string 	yes 	    User’s API token
                # path 	        string 	no 	        Server path to store the XML file
                # password 	    string 	no 	        Password used to encrypt the exported data
                "params_details": {
                    "authToken": {"type": "str", "required": True},
                    "path": {"type": "str", "required": False},
                    "password": {"type": "str", "required": False},
                },
                "json_body": {
                    "jsonrpc": "2.0",
                    "method": "config/export",
                    "params": {"authToken": None, "path": None, "password": None},
                    "id": None,
                },
            },
        }

    @property
    def api_version(self):
        if 'DD_SYSPASS_CLIENT_API_VERSION' in os.environ:
            print(
                Fore.WHITE + Style.BRIGHT + "DD_SYSPASS_CLIENT_API_VERSION: {0}".format(
                    os.environ['DD_SYSPASS_CLIENT_API_VERSION'])
            )
            return os.environ['DD_SYSPASS_CLIENT_API_VERSION']
        if self.__api_version is None:
            return syspassclient.dd.syspass["api"]["version"]
        return self.__api_version

    @api_version.setter
    def api_version(self, version=None):
        if version is None:
            if self.__api_version is not None:
                self.__api_version = None
            return

        if type(version) != str:
            raise TypeError('"version" must be a str type or None')

        if self.__api_version != version:
            self.__api_version = version

    @property
    def api_filename_ext(self):
        """
        Return the api_filename_extension property

        :return: the filename extension like '.yaml'
        :rtype: str
        """
        return self.__api_filename_ext

    @api_filename_ext.setter
    def api_filename_ext(self, extension=None):
        """
        Set the api_filename_ext property

        :param extension: a extension like '.yaml' or None for restore default one ('.yaml')
        :type extension: str or None
        :raise TypeError: When 'extension' is not a str type
        """
        if extension is None:
            extension = ".yaml"

        if not isinstance(extension, str):
            raise TypeError("'extension' must be a str type")

        if self.__api_filename_ext != extension:
            self.__api_filename_ext = extension

    @property
    def api_directory(self):
        return self.__api_directory

    @api_directory.setter
    def api_directory(self, directory=None):
        """
        Return the directory path where is store API yaml file(s)

        :param directory: in case you want force a special directory
        :type directory: str
        :return: the absolute path of the api_directory
        :rtype: str
        """
        if directory is None:
            if self.__api_directory != os.path.join(os.path.dirname(os.path.abspath(__file__)), "api"):
                self.__api_directory = os.path.join(os.path.dirname(os.path.abspath(__file__)), "api")

        else:
            if type(directory) != str:
                raise TypeError('"directory" must be a str type or None')
            if self.__api_directory != directory:
                self.__api_directory = directory

    @property
    def api_filename(self):
        """
        Return the api_filename)

        :return: the filename to load
        :rtype: str
        """
        return self.api_version + self.api_filename_ext

    @property
    def api_file(self):
        """
        Return the absolute path of the API yaml file to load

        :return: path of the API yaml file to load
        :rtype: str
        """
        return os.path.join(self.api_directory, self.api_filename)

    def read_api(self, api_file=None):
        """
        Read the API file

        :param api_file: the file path to to load
        :type api_file: str or None for the default one
        """

        if api_file is None:
            api_file = self.api_file

        with open(api_file) as f:
            self.api_data = load(f, Loader=Loader)
            f.close()
            if self.debug and self.debug_level > 1:
                print(Fore.YELLOW + Style.BRIGHT + "{0}: ".format(self.__class__.__name__.upper()), end='')
                print(Fore.WHITE + Style.BRIGHT + "{0} ".format("Load API File"))

                print(Fore.WHITE + Style.BRIGHT + "File: ", end='')
                print(api_file)

                if self.debug and self.debug_level > 2:
                    for line in dump(self.api_data).split("\n"):
                        if self.verbose:
                            print(Fore.CYAN + Style.BRIGHT + "< " + Fore.RESET + Style.RESET_ALL + str(line))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# This file is part of SysPass Client
#
# Copyright (C) 2020  DigDeo SAS
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


# Inspired by: http://code.activestate.com/recipes/65207-constants-in-python/?in=user-97991


class Constants(object):
    class ConstError(TypeError):
        pass

    def __setattr__(self, name, value):
        if name in self.__dict__:
            raise self.ConstError("Can't rebind const(%s)" % name)
        self.__dict__[name] = value

    def __getattr__(self, name):
        if name not in self.__dict__:
            raise self.ConstError("No attribute %s exist" % name)
        return self.__dict__[name]


#############################
# Variables
#############################
dd = Constants()

#############################
# Internal
#############################

#########################
# ctype-style character #
#########################

# whitespace -- a string containing all ASCII whitespace
whitespace = ' \t\n\r\v\f'

# ascii_lowercase -- a string containing all ASCII lowercase letters
ascii_lowercase = 'abcdefghijklmnopqrstuvwxyz'

# ascii_uppercase -- a string containing all ASCII uppercase letters
ascii_uppercase = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'

# ascii_letters -- a string containing all ASCII letters
ascii_letters = ascii_lowercase + ascii_uppercase

# digits -- a string containing all ASCII decimal digits
digits = '0123456789'

# hexdigits -- a string containing all ASCII hexadecimal digits
hexdigits = digits + 'abcdef' + 'ABCDEF'

# octdigits -- a string containing all ASCII octal digits
octdigits = '01234567'

# punctuation -- a string containing all ASCII punctuation characters
punctuation = "!\"#$%&'()*+,-./:;<=>?@[\\]^_`{|}~"
allowed_punctuation = "-_|./?=+()[]~*{}#"

# accent
accent = "éèàùô"

dd.whitespace = whitespace
dd.ascii_lowercase = ascii_lowercase
dd.ascii_uppercase = ascii_uppercase
dd.ascii_letters = ascii_letters
dd.digits = digits
dd.hexdigits = hexdigits
dd.octdigits = octdigits
dd.punctuation = punctuation
dd.allowed_punctuation = allowed_punctuation
dd.accent = accent
dd.ascii_all = digits + ascii_letters + punctuation + whitespace + accent

dd.syspass = {
    'api': {
        'version': '3.1',
        '3.0': {
            'methods': [
                'account/search',
                'account/view',
                'account/viewPass',
                'account/editPass',
                'account/create',
                'account/edit',
                'account/delete',
                'category/search',
                'category/view',
                'category/create',
                'category/edit',
                'category/delete',
                'client/search',
                'client/view',
                'client/create',
                'client/edit',
                'client/delete',
                'tag/search',
                'tag/view',
                'tag/create',
                'tag/edit',
                'tag/delete',
                'usergroup/search',
                'usergroup/view',
                'usergroup/create',
                'usergroup/edit',
                'usergroup/delete',
                'config/backup',
                'config/export'
            ]
        },
        '3.1': {
            'methods': [
                'account/search',
                'account/view',
                'account/viewPass',
                'account/editPass',
                'account/create',
                'account/edit',
                'account/delete',
                'category/search',
                'category/view',
                'category/create',
                'category/edit',
                'category/delete',
                'client/search',
                'client/view',
                'client/create',
                'client/edit',
                'client/delete',
                'tag/search',
                'tag/view',
                'tag/create',
                'tag/edit',
                'tag/delete',
                'userGroup/search',
                'userGroup/view',
                'userGroup/create',
                'userGroup/edit',
                'userGroup/delete',
                'config/backup',
                'config/export'
            ]
        }
    }
}
